/* catchfile.c
 *
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/

/*****************************************************************************/
/*** catchfile.c                                                           ***/
/***                                                                       ***/
/*** This is a quick tool to accept files sent from tossfile.  Please be   ***/
/*** aware that this does not check for security.  It also is incomplete.  ***/
/*** directories that are either inaccessible or non-existent will report  ***/
/*** an error instead of either getting permission or creating the needed  ***/
/*** directories.                                                          ***/
/*****************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <resolv.h>
#include <netdb.h>
#include "tosscatch.h"

/*---------------------------------------------------------------------*/
/*--- CreatePath - create the needed path for the destination.      ---*/
/*---------------------------------------------------------------------*/
int CreatePath(char *filepath)
{	char *fpath, *c;

	fpath = strdup(filepath);
	if ( (c = strrchr(fpath, '/')) != NULL )
	{
		c = 0;
		/*--Incomplete---*/
	}
	free(fpath);
	return 0;
}

/*---------------------------------------------------------------------*/
/*--- CatchFile - get filename/contents and create a local file     ---*/
/*---------------------------------------------------------------------*/
void CatchFile(int sd)
{	int fd, size, len=0;
	TFileMsg msg;

	if ( recv(sd, &msg, sizeof(msg), 0 ) < 0 )
	{
		perror("Header failure");
		return;
	}
	printf("%s\n", msg.Name);
	if ( CreatePath(msg.Name) != 0 )
	{
		perror("Filename failure");
		return;
	}
	if ( (fd = creat(msg.Name, ntohs(msg.Mode))) < 0 )
	{
		msg.Length = htonl(-1);
		send(sd, &msg, sizeof(msg), 0);
		perror(msg.Name);
		return;
	}
	send(sd, &msg, sizeof(msg), 0);
	size = ntohl(msg.Length);
	while ( len < size )
	{	char buf[32768];
		int bytes;

		bytes = recv(sd, buf, sizeof(buf), 0);
		if ( bytes < 0 )
			perror("recv()");
		else
		{
			write(fd, buf, bytes);
			len += bytes;
		}
	}
	close(fd);
	close(sd);
}

#define panic(msg)	{perror(msg);exit(1);}

/*---------------------------------------------------------------------*/
/*--- main - set up server and recieve connections.                 ---*/
/*---------------------------------------------------------------------*/
int main(int count, char *strings[])
{	int sd, port = DEFAULT_PORT;
	struct sockaddr_in addr;

	if ( count == 2 )
		port = atoi(strings[1]);
	sd = socket(PF_INET, SOCK_STREAM, 0);
	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = INADDR_ANY;
	if ( bind(sd, &addr, sizeof(addr)) != 0 )
		panic("bind()");
	if ( listen(sd, 4) != 0 )
		panic("listen()");
	while (1)
	{	int i, len=sizeof(addr);
		short numfiles;
		int client = accept(sd, &addr, &len);
		fprintf(stderr, "host(%s:%d): ", inet_ntoa(addr.sin_addr),
			ntohs(addr.sin_port));
		recv(client, &numfiles, sizeof(numfiles), 0);
		numfiles = ntohs(numfiles);
		for ( i = 0; i < numfiles; i++ )
		{
			fprintf(stderr, "%d/%d: ", i+1, numfiles);
			CatchFile(client);
		}
	}
	close(sd);
}
