/* tossfile.c
 *
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/

/*****************************************************************************/
/*** tossfile.c                                                            ***/
/***                                                                       ***/
/*** Takes the files from the command line and ships a copy to the         ***/
/*** destination.  Note that this program has no security.                 ***/
/*****************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <resolv.h>
#include <netdb.h>
#include "tosscatch.h"

/*---------------------------------------------------------------------*/
/*--- FillAddress - convert the host:port to the socket address     ---*/
/*---------------------------------------------------------------------*/
void FillAddress(struct sockaddr_in *addr, char *name)
{	int port=DEFAULT_PORT;
	char *p = strchr(name, ':');
	struct hostent *host;

	if ( p != NULL )
	{
		port = atoi(p+1);
		*p = 0;
	}
	if ( port < 1024 )
	{
		fprintf(stderr, "Illegal port number (must be >= 1024)\n");
		abort();
	}

	host = gethostbyname(name);
	memset(addr, 0, sizeof(*addr));
	addr->sin_family = AF_INET;
	addr->sin_port = htons(port);
	addr->sin_addr.s_addr = *(long*)host->h_addr;
}

/*---------------------------------------------------------------------*/
/*--- SendFile - send the file to the host.                         ---*/
/*---------------------------------------------------------------------*/
void SendFile(int sd, char *fname)
{	int fd, len=0;
	TFileMsg msg;
	struct stat stat;
	time_t t;

	if ( (fd = open(fname, O_RDONLY)) < 0 )
	{
		perror(fname);
		return;
	}
	if ( fstat(fd, &stat) < 0 )
	{
		perror(fname);
		close(fd);
		return;
	}
	memset(&msg, 0, sizeof(msg));
	msg.Length = htonl(stat.st_size);
	msg.Mode = htons(stat.st_mode);
	strncpy(msg.Name, fname, sizeof(msg.Name));
	msg.Name[sizeof(msg.Name)-1] = 0;
	if ( send(sd, &msg, sizeof(msg), 0) < 0 )
	{
		perror("can't send file");
		close(fd);
		return;
	}
	if ( recv(sd, &msg, sizeof(msg), 0) < 0  ||  ntohl(msg.Length) == -1 )
	{
		perror("can't establish file");
		close(fd);
		return;
	}
	time(&t);
	while ( len < stat.st_size )
	{	char buf[32768];
		int bytes;

		bytes = read(fd, buf, sizeof(buf));
		if ( send(sd, buf, bytes, 0) < bytes )
			printf("short buffer!\n");
		len += bytes;
	}
	printf("Secs=%d\n", time(0)-t);
	close(fd);
}

/*---------------------------------------------------------------------*/
/*--- main - open socket and send out file one at a time            ---*/
/*---------------------------------------------------------------------*/
int main(int count, char *strings[])
{	int sd;
	struct sockaddr_in addr;

	if ( count < 3 )
	{
		printf("Usage: %s <host>[:<port>] <filename> ... \n", strings[0]);
		exit(0);
	}

	sd = socket(PF_INET, SOCK_STREAM, 0);
	FillAddress(&addr, strings[1]);
	if ( connect(sd, &addr, sizeof(addr)) != 0 )
		perror("Could not connect to host");
	else
	{	int i;
		unsigned short numfiles;

		count -= 2;
		strings += 2;
		numfiles = htons((short)count);
		send(sd, &numfiles, sizeof(numfiles), 0);
		for ( i = 0; i < count; i++ )
			SendFile(sd, strings[i]);
	}
	close(sd);
}
