/* NetImage.java
 *
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/

/*****************************************************************************/
/*** NetImage.java                                                       ***/
/***                                                                       ***/
/*****************************************************************************/
import java.awt.image.*;
import java.util.*;

public class NetImage extends BufferedImage
{
	protected NetImage(ColorModel cm, WritableRaster raster, boolean isRasterPremultiplied, Hashtable properties)
	{
		super(cm, raster, isRasterPremultiplied, properties);
	}

	static public NetImage createNetImage(BufferedImage bimage)
	{
		ColorModel cm = bimage.getColorModel();
		WritableRaster wras = bimage.getRaster();
		boolean premul = bimage.isAlphaPremultiplied();
		Hashtable props = null;
		String n[] = bimage.getPropertyNames();
		if ( n != null )
		{
			props = new Hashtable();
			for ( int i = 0; i < n.length; i++ )
				props.put(n[i], bimage.getProperty(n[i]));
		}
		return new NetImage(cm, wras, premul, props);
	}

	public boolean equals(NetImage nimage)
	{
		boolean ras_ok = false;
		boolean c_ok = false;
		if ( nimage != null )
		{
			c_ok = getColorModel().equals(nimage.getColorModel());
			DataBufferInt dbuf1 = (DataBufferInt)getRaster().getDataBuffer();
			DataBufferInt dbuf2 = (DataBufferInt)nimage.getRaster().getDataBuffer();
			int banks = dbuf1.getNumBanks();
			if ( banks == dbuf2.getNumBanks() )
			{
				ras_ok = true;
				for ( int i = 0; i < banks  &&  ras_ok; i++ )
				{
					int dat1[] = dbuf1.getData(i);
					int dat2[] = dbuf2.getData(i);
					ras_ok = Arrays.equals(dat1, dat2);
				}
			}
		}
		return c_ok && ras_ok;
	}
}
