/* VMeeting.java
 *
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/

/*****************************************************************************/
/*** VMeeting.java                                                         ***/
/***                                                                       ***/
/*****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.awt.color.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;
import java.util.*;
import java.util.zip.*;

public class VMeeting extends Frame
	implements ActionListener, Runnable, WindowListener, ItemListener
{
	VMeeting()
	{
		super("MultiChat");
		addWindowListener(this);
		setSize(550,300);
		setLayout(new BorderLayout());

		Panel p = new Panel(new FlowLayout());
		p.setBackground(SystemColor.control);
		txt = new TextField(50);
		txt.addActionListener(this);
		p.add(txt);
		Button b = new Button("Send!");
		b.addActionListener(this);
		p.add(b);
		drct = new Button("Direct!");
		drct.setEnabled(false);
		drct.addActionListener(this);
		p.add(drct);
		add(p, BorderLayout.SOUTH);

		p = new Panel(new BorderLayout());
		p.setBackground(SystemColor.control);
		usrs.addItemListener(this);
		usrs.add("(no addresses)");
		p.add(usrs, BorderLayout.CENTER);
		grps = new java.awt.List();
		grps.addItemListener(this);
		grps.add("(no groups)");
		p.add(grps, BorderLayout.SOUTH);
		add(p, BorderLayout.EAST);

		add(log, BorderLayout.CENTER);

		p = new Panel(new FlowLayout());
		p.setBackground(SystemColor.control);
		p.add(new Label("Name:", Label.RIGHT));
		name = new TextField("(anon)", 6);
		p.add(name);
		p.add(new Label("Address:", Label.RIGHT));
		dest = new TextField("224.10.10.55", 9);
		p.add(dest);
		bjoin = new Button("Join");
		bjoin.addActionListener(this);
		p.add(bjoin);
		p.add(new Label("TTL:", Label.RIGHT));
		ttl = new TextField("1", 2);
		p.add(ttl);
		p.add(chview);
		add(p, BorderLayout.NORTH);
	}

	public synchronized void addSender(String user, String addr)
	{
		if ( new_usrs )
		{
			usrs.removeAll();
			new_usrs = false;
		}
		String l[] = usrs.getItems();
		int i;

		for ( i = 0; i < l.length; i++ )
			if ( l[i].endsWith(addr) )
				break;
		if ( i >= l.length )
			usrs.add(user+addr);
		else
			usrs.replaceItem(user+addr, i);
		usrs.select(-1);
		drct.setEnabled(false);
	}

	public void run()
	{	Hashtable clients = new Hashtable();

		while (true)
		{
			byte buf[] = new byte[65536];
			DatagramPacket pkt = new DatagramPacket(buf, buf.length);
			try
			{
				mcast.receive(pkt);
				String src = pkt.getAddress().getHostAddress();
				ByteArrayInputStream ib = new ByteArrayInputStream(pkt.getData());
				GZIPInputStream iz = new GZIPInputStream(ib);
				ObjectInputStream i = new ObjectInputStream(iz);
				String sender = i.readObject().toString();
				Integer iv = (Integer)i.readObject();
				switch ( iv.intValue() )
				{
					case 0:
					{
						String msg = (String)i.readObject();
						addSender(sender, " ("+src+")");
						if ( log.getText().length() > 1 )
							log.append("\n");
						log.append(sender+"("+src+"): "+msg);
						break;
					}
					case 1:
					{
						iv = (Integer)i.readObject();
						int r = iv.intValue();
						iv = (Integer)i.readObject();
						int c = iv.intValue();
						JPEGImageDecoder jpeg = JPEGCodec.createJPEGDecoder(i);
						BufferedImage im = jpeg.decodeAsBufferedImage();
						Object o = clients.get(pkt.getAddress().toString());
						if ( o == null )
						{
							MultiView mv = new MultiView(sender+"("+src+")", im, r, c);
							mv.show();
							clients.put(pkt.getAddress().toString(), mv);
						}
						else
						{
							MultiView mv = (MultiView)o;
							mv.setImage(im, r, c);
						}
						break;
					}
				}
			}
			catch (Exception err)
			{
				System.err.println("Listener: "+err.toString());
				System.exit(0);
			}
		}
	}

	public void windowActivated(WindowEvent e)		{}
	public void windowClosed(WindowEvent e)			{}
	public void windowClosing(WindowEvent e)		{System.exit(0);}
	public void windowDeactivated(WindowEvent e)	{}
	public void windowDeiconified(WindowEvent e)	{}
	public void windowIconified(WindowEvent e)		{}
	public void windowOpened(WindowEvent e)			{}

	public void itemStateChanged(ItemEvent e)
	{
		if ( e.getItem() == usrs )
		{
			String s = usrs.getSelectedItem();
			dest.setText(s.substring(s.lastIndexOf('(')+1, s.lastIndexOf(')')));
			if ( usrs.getSelectedIndex() != -1 )
				drct.setEnabled(true);
			else
				drct.setEnabled(false);
		}
		else if ( e.getItem() == grps )
		{
			dest.setText(grps.getSelectedItem());
		}
		try
		{
			bjoin.setEnabled(InetAddress.getByName(dest.getText()).isMulticastAddress());
		}
		catch (Exception err)
		{
			System.err.println("Item changed: "+err.toString());
		}
	}

	synchronized static public void transmitMessage(byte msg[], String action) throws UnknownHostException, IOException
	{
		InetAddress addr;
		if ( action.equals("Direct!") )
		{
			String s = usrs.getSelectedItem();
			String a = s.substring(s.lastIndexOf('(')+1, s.lastIndexOf(')'));
			addr = InetAddress.getByName(a);
		}
		else
			addr = InetAddress.getByName(dest.getText());
		DatagramPacket pkt = new DatagramPacket(msg, msg.length, addr, port);
		Integer ttlval = new Integer(ttl.getText());
		mcast.send(pkt, (byte)ttlval.intValue());
	}

	public void actionPerformed(ActionEvent e)
	{
		try
		{
			if ( e.getActionCommand().equals("Join") )
			{
				int i;
				String l[] = grps.getItems();
				for ( i = 0; i < l.length; i++ )
					if ( l[i].equals(dest.getText()) )
						break;
				if ( i >= l.length )
				{
					if ( new_grps )
					{
						grps.removeAll();
						new_grps = false;
					}
					InetAddress group = InetAddress.getByName(dest.getText());
					mcast.joinGroup(group);
					grps.add(dest.getText());
				}
			}
			else
			{
				ByteArrayOutputStream ob = new ByteArrayOutputStream();
				GZIPOutputStream oz = new GZIPOutputStream(ob);
				ObjectOutputStream o = new ObjectOutputStream(oz);
				o.writeObject(name.getText());
				Integer i = new Integer(0);
				o.writeObject(i);
				o.writeObject(txt.getText());
				o.flush();
				oz.finish();
				txt.setText("");
				transmitMessage(ob.toByteArray(), e.getActionCommand());
			}
		}
		catch (Exception err)
		{
			System.err.println("Speaker: "+err.toString());
		}
	}

	public static void main(String args[])
	{
		log = new TextArea();
		log.setEditable(false);
		usrs = new java.awt.List();
		chview = new Checkbox("Share Desktop", false);
		try
		{
			mcast = new MulticastSocket(port);
			VMeeting vm = new VMeeting();
			Thread t = new Thread(vm);
			Thread vs = new ViewServer();
			vm.show();
			t.start();
//			vs.start();
		}
		catch (Exception err)
		{
			System.err.println("Main: "+err.toString());
		}
	}

	static final int port=11220;
	static TextArea log;
	static java.awt.List usrs;
	static boolean new_usrs = true;
	static java.awt.List grps;
	static boolean new_grps = true;
	static MulticastSocket mcast;
	static TextField name;
	static TextField dest;
	static TextField ttl;
	static Checkbox chview;
	TextField txt;
	Button drct;
	Button bjoin;
}

//class Channel
//{
//	public Channel(boolean isStream)
//	{
//	}
//
//	public void join(