/*
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/
#include <stdio.h>
#include <sys/socket.h>
#include "tools.h"

/*********************************************************************/
/*** SocketDatagram                                                ***/
/***                                                               ***/
/*** Create datagram socket on a port and an interface (optional). ***/
/*** If any error, return negative value. Returns socket           ***/
/*** descriptor, if successful.                                    ***/
/*********************************************************************/
int SocketDatagram(int port, const char *iface)
{	int sd;
	struct sockaddr_in addr;

	sd = socket(PF_INET, SOCK_DGRAM, 0);
	if ( sd >= 0 )
	{
		if ( GetAddress(&addr, iface, port) == 0 )
		{
			if ( bind(sd, &addr, sizeof(addr)) != 0 )
			{
				perror("Bind port failed");
				close(sd);
				sd = -1;
			}
		}
	}
	return sd;
}

/*********************************************************************/
/*** JoinGroup                                                     ***/
/***                                                               ***/
/*** Create datagram socket on a port and an interface (optional). ***/
/*** If any error, return negative value. Returns socket           ***/
/*** descriptor, if successful.                                    ***/
/*********************************************************************/
int JoinGroup(int sd, const char *group)
{	int sd;
	struct sockaddr_in addr;

	sd = socket(PF_INET, SOCK_DGRAM, 0);
	if ( sd >= 0 )
	{
		if ( GetAddress(&addr, iface, port) == 0 )
		{
			if ( bind(sd, &addr, sizeof(addr)) != 0 )
			{
				perror("Bind port failed");
				close(sd);
				sd = -1;
			}
		}
	}
	return sd;
}

