/*
 * Copyright (c) 2000 Sean Walton and Macmillan Publishers.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/
#include <stdio.h>
#include <sys/socket.h>
#include <netdb.h>
#include "tools.h"

/*********************************************************************/
/*** GetService                                                    ***/
/***                                                               ***/
/*** Using the /etc/services file, lookup the appropriate port     ***/
/*** number.  Returns negative if failure.                         ***/
/*********************************************************************/
int GetService(const char *service, const char *protocol)
{	struct servent *serv;

	serv = getservbyname(service, protocol);
	if ( serv != NULL )
		return serv->s_port;
	else
	{
		perror(service);
		return -1;
	}
}

/*********************************************************************/
/*** GetHost                                                       ***/
/***                                                               ***/
/*** Converts hostname to IP address.                              ***/
/*********************************************************************/


/*********************************************************************/
/*** GetAddress                                                    ***/
/***                                                               ***/
/*** Use the DNS to resolve host name.  (If "host" is an IP        ***/
/*** address, no resolution is done, and the address is converted  ***/
/*** for you.)  Fills the sockaddr_in struct.                      ***/
/*** Returns zero if successful; negative value, otherwise.        ***/
/*********************************************************************/
int GetAddress(struct sockaddr_in *addr, const char *host, int port)
{	struct hostent *ent;

	bzero(addr, sizeof(*addr));
	addr->sin_family = AF_INET;
	addr->sin_port = htons(port);
	if ( host != 0 )
	{
		ent = gethostbyname(host);
		if ( ent != NULL )
		{
			addr->sin_addr.s_addr = *(long*)(ent->h_addr);
			return 0;
		}
		perror(host);
		return -1;
	}
	else
		addr->sin_addr.s_addr = INADDR_ANY;
	return 0;
}

/*********************************************************************/
/*** GetAddressSvr                                                 ***/
/***                                                               ***/
/*** Use DNS and Servent services to resolve "host" and "service"  ***/
/*** (similar to GetAddress()).                                    ***/
/*** Returns zero if successful; negative value, otherwise.        ***/
/*********************************************************************/
int GetAddressSvc(struct sockaddr_in *addr, const char *host, const char *service)
{	int port;
	struct hostent *ent;

	bzero(addr, sizeof(*addr));
	addr->sin_family = AF_INET;
	port = GetService(service, "TCP");
	if ( port > 0 )
		addr->sin_port = port;
	if ( host != 0 )
	{
		ent = gethostbyname(host);
		if ( ent != NULL )
		{
			addr->sin_addr.s_addr = *(long*)(ent->h_addr);
			return 0;
		}
		else
		{
			perror(service);
			return -1;
		}
	}
	else
		addr->sin_addr.s_addr = INADDR_ANY;
	return 0;
}
